//
//  RateView.m
//
//  Based on code by Ray Wenderlich on 2/17/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//
//	B4i Library by Henrik Skov (henry.wood.dk@gmail.com)

#import "RateView.h"

@implementation RateView

@synthesize NotSelectedImage = _notSelectedImage;
@synthesize HalfSelectedImage = _halfSelectedImage;
@synthesize FullSelectedImage = _fullSelectedImage;
@synthesize Rating = _rating;
@synthesize Enabled = _enabled;
@synthesize ImageViews = _imageViews;
@synthesize MaxRating = _maxRating;
@synthesize MidMargin = _midMargin;
@synthesize LeftMargin = _leftMargin;
@synthesize MinImageSize = _minImageSize;
//@synthesize delegate = _delegate;

- (void)baseInit {

	// Defaults
    _notSelectedImage = nil;
    _halfSelectedImage = nil;
    _fullSelectedImage = nil;
    _rating = 0;
    _enabled = NO;    
    _imageViews = [[NSMutableArray alloc] init];
    _maxRating = 5;
    _midMargin = 5;
    _leftMargin = 0;
    _minImageSize = CGSizeMake(5, 5);
    _delegate = nil;
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self baseInit];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {

    if ((self = [super initWithCoder:aDecoder])) {
        [self baseInit];
    }
    return self;
}

- (void)refresh {

    for(int i = 0; i < self.imageViews.count; ++i) {
	
        UIImageView *imageView = [self.ImageViews objectAtIndex:i];
		
        if (self.Rating >= i+1) {
            imageView.image = self.FullSelectedImage;
        } else if (self.Rating > i) {
            imageView.image = self.HalfSelectedImage;
        } else {
            imageView.image = self.NotSelectedImage;
        }
    }
}

- (void)layoutSubviews {

    [super layoutSubviews];
    
    if (self.NotSelectedImage == nil) return;
    
    float desiredImageWidth = (self.frame.size.width - (self.LeftMargin*2) - (self.MidMargin*self.ImageViews.count)) / self.ImageViews.count;
    float imageWidth = MAX(self.MinImageSize.width, desiredImageWidth);
    float imageHeight = MAX(self.MinImageSize.height, self.frame.size.height);
    
    for (int i = 0; i < self.ImageViews.count; ++i) {
        
        UIImageView *imageView = [self.ImageViews objectAtIndex:i];
        CGRect imageFrame = CGRectMake(self.LeftMargin + i*(self.MidMargin+imageWidth), 0, imageWidth, imageHeight);
        imageView.frame = imageFrame;
        
    }    
    
}

- (void)setEnabled:(BOOL)enabled {
    _enabled = enabled;
}

- (void)setMaxRating:(int)maxRating {

    _maxRating = maxRating;
    
    // Remove old image views
    for(int i = 0; i < self.imageViews.count; ++i) {
	
        UIImageView *imageView = (UIImageView *) [self.ImageViews objectAtIndex:i];
        [imageView removeFromSuperview];
    }
    [self.ImageViews removeAllObjects];
    
    // Add new image views
    for(int i = 0; i < maxRating; ++i) {
	
        UIImageView *imageView = [[UIImageView alloc] init];
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        [self.imageViews addObject:imageView];
        [self addSubview:imageView];
    }
    
    // Relayout and refresh
    [self setNeedsLayout];
    [self refresh];
}

- (void)setNotSelectedImage:(UIImage *)image {

    _notSelectedImage = image;
    [self refresh];
}

- (void)setHalfSelectedImage:(UIImage *)image {

    _halfSelectedImage = image;
    [self refresh];
}

- (void)setFullSelectedImage:(UIImage *)image {

    _fullSelectedImage = image;
    [self refresh];
}

- (void)setRating:(float)rating {

    _rating = rating;
    [self refresh];
}

- (void)handleTouchAtLocation:(CGPoint)touchLocation {

    if (!self.Enabled) return;
    
    int newRating = 0;
	
    for(int i = self.ImageViews.count - 1; i >= 0; i--) {
        UIImageView *imageView = [self.ImageViews objectAtIndex:i];        
        if (touchLocation.x > imageView.frame.origin.x) {
            newRating = i+1;
            break;
        }
    }
    
    self.Rating = newRating;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint touchLocation = [touch locationInView:self];
    [self handleTouchAtLocation:touchLocation];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint touchLocation = [touch locationInView:self];
    [self handleTouchAtLocation:touchLocation];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    
	//[self.delegate rateView:self ratingDidChange:self.Rating];

	// Raise B4i event
	[B4IObjectWrapper raiseEvent:self :@"_ratingchanged:" :@[@((double)self.Rating)]];

}

#define OBJ ((RateView *) self.object)

+(Class)getClass {
    return [RateView class];
}

-(void)Initialize:(B4I*)bi :(NSString *)EventName {

	[B4IObjectWrapper setBIAndEventName:self :bi :EventName];
}

@end
